//SETA43 15/07/2024
// seta43.duckdns.org
// seta43.blogspot.com
//BASADO EN: https://www.hackster.io/earlephilhower/esp8266-digital-radio-ee747f
//Version A

// To run, set your ESP8266 build to 160MHz, update the SSID info, and upload.

//#define DEBUG 1

#include <Arduino.h>

#ifdef ESP32
    #include <WiFi.h>
#else
    #include <ESP8266WiFi.h>
#endif
#include "AudioFileSourceICYStream.h"
#include "AudioFileSourceBuffer.h"
#include "AudioGeneratorMP3.h"
#include "AudioOutputI2SNoDAC.h"

// Enter your WiFi setup here:
const char *SSID = "MiFibra";
const char *PASSWORD = "contraseña";

AudioGeneratorMP3 *mp3;
AudioFileSourceICYStream *file;
AudioFileSourceBuffer *buff;
AudioOutputI2SNoDAC *out;

#define D0 16
#define D1 5
#define D2 4
#define D3 0
#define D4 2
#define D5 14
#define D6 12
#define D7 13
#define D8 15


struct EMI {
    char numero;
    char nombre[15];
    char n_url[200];
};

struct EMI emisora[10]={ {0,"SER","http://22193.live.streamtheworld.com/CADENASER_SC"},
        {1,"CADENA100", "http://cadena100-cope-rrcast.flumotion.com/cope/cadena100-low.mp3"},
        {2,"CADENAdial","http://playerservices.streamtheworld.com/api/livestream-redirect/CADENADIAL.mp3"},
        {3,"COPE", "http://net1-cope-rrcast.flumotion.com/cope/net1-low.mp3"},
        {4,"ESRADIO", "http://libertaddigital-radio-live1.flumotion.com/libertaddigital/ld-live1-low.mp3"},
        {5,"SPAIN","http://91.121.155.204:8022/stream/1/"},
        {6,"LATINO", "http://91.121.38.100:8410/;stream/1"},
        {7,"Country", "http://46.105.125.110:13656/stream/1/"} };


#define bit_4 D7
#define bit_2 D6
#define bit_1 D5

#define PIN_PLAY D1
#define PIN_BOTON D2

int canal;
bool estado;


void setup()
{
  Serial.begin(115200);
  while (!Serial);
  Serial.println("----------");
  Serial.print("Conectando a WiFi->"); Serial.println(SSID);

  WiFi.disconnect();
  WiFi.softAPdisconnect(true);
  WiFi.mode(WIFI_STA); 
  WiFi.begin(SSID, PASSWORD);

  // Try forever
  while (WiFi.status() != WL_CONNECTED) {  Serial.print("..");  delay(500);  }
  Serial.print("Conectado a "); Serial.println(SSID);

  pinMode(PIN_PLAY,OUTPUT);
  pinMode(PIN_BOTON,INPUT_PULLUP);
  
  pinMode(bit_1,INPUT_PULLUP);
  pinMode(bit_2,INPUT_PULLUP);
  pinMode(bit_4,INPUT_PULLUP);
  
  listEmisoras();
    
  file = NULL;
  buff = NULL;
  out = new AudioOutputI2SNoDAC();
  mp3= NULL;
   
  
  delay(1000);
  StartStream();
  digitalWrite(PIN_PLAY, HIGH);
  estado=1;
}


void loop()
{
static int lastms = 0;
  
 if(!digitalRead(PIN_BOTON))
    {
      while(!digitalRead(PIN_BOTON)){ Serial.print("."); delay(200);}
      boton_D2();
    }

  if ( estado==1 && mp3->isRunning() ) 
  {
    if (!mp3->loop()) { mp3->stop(); digitalWrite(PIN_PLAY, 0); }
  } 
  
}

void boton_D2(void)
{
  if(estado==1)
    {   Serial.println("BOTON STOP");  StopStream();   }
  else
    {   Serial.println("BOTON PLAY");   StartStream(); }
  
  digitalWrite(PIN_PLAY, estado);  
}

void listEmisoras()
{
for(int z=0;z<8;z++)
    {
      Serial.printf("%d  %s  %s\n", z,emisora[z].nombre,emisora[z].n_url);
      Serial.flush();
    }   
}

void StopStream()
{
  if (mp3) {   mp3->stop();   delay(300);  delete mp3;    mp3 = NULL;   } 
  if (buff) {  buff->close();  delay(300); delete buff;   buff = NULL;  }
  if (file) {  file->close();  delay(300); delete file;   file = NULL;  } 
  estado=0;
  
  listEmisoras();
}

void StartStream()
{
  canal=0;
  if(digitalRead(bit_1)) canal=1;
  if(digitalRead(bit_2)) canal+=2;
  if(digitalRead(bit_4)) canal+=4;
  Serial.printf("CANAL=%d  EMISORA=%s\n",canal,emisora[canal].nombre);  Serial.flush();

  file = new AudioFileSourceICYStream(emisora[canal].n_url);
  buff = new AudioFileSourceBuffer(file, 4096);  
  mp3 = new AudioGeneratorMP3();
  mp3->begin(buff, out);

#ifdef DEBUG
audioLogger = &Serial;
  file->RegisterMetadataCB(MDCallback, (void*)"ICY");
  buff->RegisterStatusCB(StatusCallback, (void*)"buffer");
  mp3->RegisterStatusCB(StatusCallback, (void*)"mp3");
#endif
  
  estado=1;
}





//_____________________________________________________________
#ifdef DEBUG
// Called when a metadata event occurs (i.e. an ID3 tag, an ICY block, etc.
void MDCallback(void *cbData, const char *type, bool isUnicode, const char *string)
{
  const char *ptr = reinterpret_cast<const char *>(cbData);
  (void) isUnicode; // Punt this ball for now
  // Note that the type and string may be in PROGMEM, so copy them to RAM for printf
  char s1[32], s2[64];
  strncpy_P(s1, type, sizeof(s1));
  s1[sizeof(s1)-1]=0;
  strncpy_P(s2, string, sizeof(s2));
  s2[sizeof(s2)-1]=0;
  Serial.printf("METADATA(%s) '%s' = '%s'\n", ptr, s1, s2);
  Serial.flush();
}

// Called when there's a warning or error (like a buffer underflow or decode hiccup)
void StatusCallback(void *cbData, int code, const char *string)
{
  if(code!=257)
  {
    const char *ptr = reinterpret_cast<const char *>(cbData);
    // Note that the string may be in PROGMEM, so copy it to RAM for printf
    char s1[64];
    strncpy_P(s1, string, sizeof(s1));
    s1[sizeof(s1)-1]=0;
    Serial.printf("STATUS(%s) '%d' = '%s'\n", ptr, code, s1);
    Serial.flush();
  }
}
#endif
//_____________________________________________________________
